@@ -23,13 +23,16 @@ Some slight modifications, but at some point
 we should look at maybe pushing this up to Oslo
 """
 
+from __future__ import annotations  # Remove when only supporting python 3.9+
 
 import contextlib
 import errno
+import io
 import math
 import os
 import re
 import tempfile
+from typing import ContextManager, Generator, Optional  # noqa: H301
 
 import cryptography
 from cursive import exception as cursive_exception
@@ -43,8 +46,8 @@ from oslo_utils import imageutils
 from oslo_utils import timeutils
 from oslo_utils import units
 import psutil
-import six
 
+from cinder import context
 from cinder import exception
 from cinder.i18n import _
 from cinder.image import accelerator
@@ -70,6 +73,22 @@ image_opts = [
     cfg.IntOpt('image_conversion_address_space_limit',
                default=1,
                help='Address space limit in gigabytes to convert the image'),
+    cfg.BoolOpt('image_conversion_disable',
+                default=False,
+                help='Disallow image conversion when creating a volume from '
+                'an image and when uploading a volume as an image. Image '
+                'conversion consumes a large amount of system resources and '
+                'can cause performance problems on the cinder-volume node. '
+                'When set True, this option disables image conversion.'),
+    cfg.ListOpt('vmdk_allowed_types',
+                default=['streamOptimized', 'monolithicSparse'],
+                help='A list of strings describing the VMDK createType '
+                'subformats that are allowed.  We recommend that you only '
+                'include single-file-with-sparse-header variants to avoid '
+                'potential host file exposure when processing named extents '
+                'when an image is converted to raw format as it is written '
+                'to a volume.  If this list is empty, no VMDK images are '
+                'allowed.'),
 ]
 
 CONF = cfg.CONF
@@ -95,7 +114,8 @@ QEMU_IMG_MIN_CONVERT_LUKS_VERSION = '2.10'
 COMPRESSIBLE_IMAGE_FORMATS = ('qcow2',)
 
 
-def validate_stores_id(context, image_service_store_id):
+def validate_stores_id(context: context.RequestContext,
+                       image_service_store_id: str) -> None:
     image_service = glance.get_default_image_service()
     stores_info = image_service.get_stores(context)['stores']
     for info in stores_info:
@@ -108,21 +128,23 @@ def validate_stores_id(context, image_service_store_id):
     raise exception.GlanceStoreNotFound(store_id=image_service_store_id)
 
 
-def fixup_disk_format(disk_format):
+def fixup_disk_format(disk_format: str) -> str:
     """Return the format to be provided to qemu-img convert."""
 
     return QEMU_IMG_FORMAT_MAP.get(disk_format, disk_format)
 
 
-def from_qemu_img_disk_format(disk_format):
+def from_qemu_img_disk_format(disk_format: str) -> str:
     """Return the conventional format derived from qemu-img format."""
 
     return QEMU_IMG_FORMAT_MAP_INV.get(disk_format, disk_format)
 
 
-def qemu_img_info(path, run_as_root=True, force_share=False):
+def qemu_img_info(path: str,
+                  run_as_root: bool = True,
+                  force_share: bool = False) -> imageutils.QemuImgInfo:
     """Return an object containing the parsed output from qemu-img info."""
-    cmd = ['env', 'LC_ALL=C', 'qemu-img', 'info']
+    cmd = ['env', 'LC_ALL=C', 'qemu-img', 'info', '--output=json']
     if force_share:
         if qemu_img_supports_force_share():
             cmd.append('--force-share')
@@ -136,7 +158,7 @@ def qemu_img_info(path, run_as_root=True, force_share=False):
         cmd = cmd[2:]
     out, _err = utils.execute(*cmd, run_as_root=run_as_root,
                               prlimit=QEMU_IMG_LIMITS)
-    info = imageutils.QemuImgInfo(out)
+    info = imageutils.QemuImgInfo(out, format='json')
 
     # From Cinder's point of view, any 'luks' formatted images
     # should be treated as 'raw'.
@@ -146,7 +168,7 @@ def qemu_img_info(path, run_as_root=True, force_share=False):
     return info
 
 
-def get_qemu_img_version():
+def get_qemu_img_version() -> Optional[list[int]]:
     """The qemu-img version will be cached until the process is restarted."""
 
     global QEMU_IMG_VERSION
@@ -163,16 +185,25 @@ def get_qemu_img_version():
     return QEMU_IMG_VERSION
 
 
-def qemu_img_supports_force_share():
-    return get_qemu_img_version() >= [2, 10, 0]
+def qemu_img_supports_force_share() -> bool:
+    ver = get_qemu_img_version()
+    if ver is None:
+        return False
+    else:
+        return ver >= QEMU_IMG_MIN_FORCE_SHARE_VERSION
 
 
-def _get_qemu_convert_luks_cmd(src, dest, out_format, src_format=None,
-                               out_subformat=None, cache_mode=None,
-                               prefix=None, cipher_spec=None,
-                               passphrase_file=None,
-                               src_passphrase_file=None):
-
+def _get_qemu_convert_luks_cmd(src: str,
+                               dest: str,
+                               out_format: str,
+                               src_format: Optional[str] = None,
+                               out_subformat: Optional[str] = None,
+                               cache_mode: Optional[str] = None,
+                               prefix: Optional[tuple] = None,
+                               cipher_spec: Optional[dict] = None,
+                               passphrase_file: Optional[str] = None,
+                               src_passphrase_file: Optional[str] = None) \
+        -> list[str]:
     cmd = ['qemu-img', 'convert']
 
     if prefix:
@@ -196,12 +227,18 @@ def _get_qemu_convert_luks_cmd(src, dest, out_format, src_format=None,
     return command
 
 
-def _get_qemu_convert_cmd(src, dest, out_format, src_format=None,
-                          out_subformat=None, cache_mode=None,
-                          prefix=None, cipher_spec=None,
-                          passphrase_file=None, compress=False,
-                          src_passphrase_file=None):
-
+def _get_qemu_convert_cmd(src: str,
+                          dest: str,
+                          out_format: str,
+                          src_format: Optional[str] = None,
+                          out_subformat: Optional[str] = None,
+                          cache_mode: Optional[str] = None,
+                          prefix: Optional[tuple] = None,
+                          cipher_spec: Optional[dict] = None,
+                          passphrase_file: Optional[str] = None,
+                          compress: bool = False,
+                          src_passphrase_file: Optional[str] = None) \
+        -> list[str]:
     if src_passphrase_file is not None:
         if passphrase_file is None:
             message = _("Can't create unencrypted volume %(format)s "
@@ -247,7 +284,8 @@ def _get_qemu_convert_cmd(src, dest, out_format, src_format=None,
     # fix the encrypted case.
 
     if (src_format or '').lower() not in ('', 'ami'):
-        cmd += ('-f', src_format)  # prevent detection of format
+        assert src_format is not None
+        cmd += ['-f', src_format]  # prevent detection of format
 
     # NOTE(lyarwood): When converting to LUKS add the cipher spec if present
     # and create a secret for the passphrase, written to a temp file
@@ -266,14 +304,15 @@ def _get_qemu_convert_cmd(src, dest, out_format, src_format=None,
     return cmd
 
 
-def _get_version_from_string(version_string):
+def _get_version_from_string(version_string: str) -> list[int]:
     return [int(x) for x in version_string.split('.')]
 
 
-def check_qemu_img_version(minimum_version):
+def check_qemu_img_version(minimum_version: str) -> None:
     qemu_version = get_qemu_img_version()
     if (qemu_version is None
        or qemu_version < _get_version_from_string(minimum_version)):
+        current_version: Optional[str]
         if qemu_version:
             current_version = '.'.join((str(element)
                                        for element in qemu_version))
@@ -287,11 +326,17 @@ def check_qemu_img_version(minimum_version):
         raise exception.VolumeBackendAPIException(data=_msg)
 
 
-def _convert_image(prefix, source, dest, out_format,
-                   out_subformat=None, src_format=None,
-                   run_as_root=True, cipher_spec=None,
-                   passphrase_file=None, compress=False,
-                   src_passphrase_file=None):
+def _convert_image(prefix: tuple,
+                   source: str,
+                   dest: str,
+                   out_format: str,
+                   out_subformat: Optional[str] = None,
+                   src_format: Optional[str] = None,
+                   run_as_root: bool = True,
+                   cipher_spec: Optional[dict] = None,
+                   passphrase_file: Optional[str] = None,
+                   compress: bool = False,
+                   src_passphrase_file: Optional[str] = None) -> None:
     """Convert image to other format.
 
     NOTE: If the qemu-img convert command fails and this function raises an
@@ -321,6 +366,7 @@ def _convert_image(prefix, source, dest, out_format,
     # flush properly and more efficiently than would be done
     # setting O_DIRECT, so check for that and skip the
     # setting for non BLK devs
+    cache_mode: Optional[str]
     if (utils.is_blk_device(dest) and
             volume_utils.check_for_odirect_support(source,
                                                    dest,
@@ -388,10 +434,45 @@ def _convert_image(prefix, source, dest, out_format,
     LOG.info(msg, {"sz": fsz_mb, "mbps": mbps})
 
 
-def convert_image(source, dest, out_format, out_subformat=None,
-                  src_format=None, run_as_root=True, throttle=None,
-                  cipher_spec=None, passphrase_file=None,
-                  compress=False, src_passphrase_file=None):
+def convert_image(source: str,
+                  dest: str,
+                  out_format: str,
+                  out_subformat: Optional[str] = None,
+                  src_format: Optional[str] = None,
+                  run_as_root: bool = True,
+                  throttle=None,
+                  cipher_spec: Optional[dict] = None,
+                  passphrase_file: Optional[str] = None,
+                  compress: bool = False,
+                  src_passphrase_file: Optional[str] = None,
+                  image_id: Optional[str] = None,
+                  data: Optional[imageutils.QemuImgInfo] = None) -> None:
+    """Convert image to other format.
+
+    NOTE: If the qemu-img convert command fails and this function raises an
+    exception, a non-empty dest file may be left in the filesystem.
+    It is the responsibility of the caller to decide what to do with this file.
+
+    :param source: source filename
+    :param dest: destination filename
+    :param out_format: output image format of qemu-img
+    :param out_subformat: output image subformat
+    :param src_format: source image format (use image_utils.fixup_disk_format()
+           to translate from a Glance format to one recognizable by qemu_img)
+    :param run_as_root: run qemu-img as root
+    :param throttle: a cinder.throttling.Throttle object, or None
+    :param cipher_spec: encryption details
+    :param passphrase_file: filename containing luks passphrase
+    :param compress: compress w/ qemu-img when possible (best effort)
+    :param src_passphrase_file: filename containing source volume's
+                                luks passphrase
+    :param image_id: the image ID if this is a Glance image, or None
+    :param data: a imageutils.QemuImgInfo object from this image, or None
+    :raises ImageUnacceptable: when the image fails some format checks
+    :raises ProcessExecutionError: when something goes wrong during conversion
+    """
+    check_image_format(source, src_format, image_id, data, run_as_root)
+
     if not throttle:
         throttle = throttling.Throttle.get_default()
     with throttle.subcommand(source, dest) as throttle_cmd:
@@ -407,13 +488,20 @@ def convert_image(source, dest, out_format, out_subformat=None,
                        src_passphrase_file=src_passphrase_file)
 
 
-def resize_image(source, size, run_as_root=False):
+def resize_image(source: str,
+                 size: int,
+                 run_as_root: bool = False,
+                 file_format: Optional[str] = None) -> None:
     """Changes the virtual size of the image."""
-    cmd = ('qemu-img', 'resize', source, '%sG' % size)
+    cmd: tuple[str, ...]
+    if file_format:
+        cmd = ('qemu-img', 'resize', '-f', file_format, source, '%sG' % size)
+    else:
+        cmd = ('qemu-img', 'resize', source, '%sG' % size)
     utils.execute(*cmd, run_as_root=run_as_root)
 
 
-def _verify_image(img_file, verifier):
+def _verify_image(img_file: io.RawIOBase, verifier) -> None:
     # This methods must be called from a native thread, as the file I/O may
     # not yield to other greenthread in some cases, and since the update and
     # verify operations are CPU bound there would not be any yielding either,
@@ -426,7 +514,10 @@ def _verify_image(img_file, verifier):
     verifier.verify()
 
 
-def verify_glance_image_signature(context, image_service, image_id, path):
+def verify_glance_image_signature(context: context.RequestContext,
+                                  image_service: glance.GlanceImageService,
+                                  image_id: str,
+                                  path: str) -> bool:
     verifier = None
     image_meta = image_service.show(context, image_id)
     image_properties = image_meta.get('properties', {})
@@ -480,15 +571,19 @@ def verify_glance_image_signature(context, image_service, image_id, path):
                     message = _('Failed to verify signature for '
                                 'image: %(image)s due to '
                                 'error: %(error)s ') % {'image': image_id,
-                                                        'error':
-                                                            six.text_type(ex)}
+                                                        'error': ex}
                     LOG.error(message)
                     raise exception.ImageSignatureVerificationException(
                         reason=message)
     return False
 
 
-def fetch(context, image_service, image_id, path, _user_id, _project_id):
+def fetch(context: context.RequestContext,
+          image_service: glance.GlanceImageService,
+          image_id: str,
+          path: str,
+          _user_id,
+          _project_id) -> None:
     # TODO(vish): Improve context handling and add owner and auth data
     #             when it is added to glance.  Right now there is no
     #             auth checking in glance, so we assume that access was
@@ -533,8 +628,12 @@ def fetch(context, image_service, image_id, path, _user_id, _project_id):
     LOG.info(msg, {"sz": fsz_mb, "mbps": mbps})
 
 
-def get_qemu_data(image_id, has_meta, disk_format_raw, dest, run_as_root,
-                  force_share=False):
+def get_qemu_data(image_id: str,
+                  has_meta: bool,
+                  disk_format_raw: bool,
+                  dest: str,
+                  run_as_root: bool,
+                  force_share: bool = False) -> imageutils.QemuImgInfo:
     # We may be on a system that doesn't have qemu-img installed.  That
     # is ok if we are working with a RAW image.  This logic checks to see
     # if qemu-img is installed.  If not we make sure the image is RAW and
@@ -568,9 +667,102 @@ def get_qemu_data(image_id, has_meta, disk_format_raw, dest, run_as_root,
     return data
 
 
-def fetch_verify_image(context, image_service, image_id, dest,
-                       user_id=None, project_id=None, size=None,
-                       run_as_root=True):
+def check_vmdk_image(image_id: str, data: imageutils.QemuImgInfo) -> None:
+    """Check some rules about VMDK images.
+
+    Make sure the VMDK subformat (the "createType" in vmware docs)
+    is one that we allow as determined by the 'vmdk_allowed_types'
+    configuration option.  The default set includes only types that
+    do not reference files outside the VMDK file, which can otherwise
+    be used in exploits to expose host information.
+
+    :param image_id: the image id
+    :param data: an imageutils.QemuImgInfo object
+    :raises ImageUnacceptable: when the VMDK createType is not in the
+                               allowed list
+    """
+    allowed_types = CONF.vmdk_allowed_types
+
+    if not len(allowed_types):
+        msg = _('Image is a VMDK, but no VMDK createType is allowed')
+        raise exception.ImageUnacceptable(image_id=image_id, reason=msg)
+
+    try:
+        create_type = data.format_specific['data']['create-type']
+    except KeyError:
+        msg = _('Unable to determine VMDK createType')
+        raise exception.ImageUnacceptable(image_id=image_id, reason=msg)
+    except TypeError:
+        msg = _('Unable to determine VMDK createType as no format-specific '
+                'information is available')
+        raise exception.ImageUnacceptable(image_id=image_id, reason=msg)
+
+    if create_type not in allowed_types:
+        LOG.warning('Refusing to process VMDK file with createType of %r '
+                    'which is not in allowed set of: %s', create_type,
+                    ','.join(allowed_types))
+        msg = _('Invalid VMDK create-type specified')
+        raise exception.ImageUnacceptable(image_id=image_id, reason=msg)
+
+
+def check_image_format(source: str,
+                       src_format: Optional[str] = None,
+                       image_id: Optional[str] = None,
+                       data: Optional[imageutils.QemuImgInfo] = None,
+                       run_as_root: bool = True) -> None:
+    """Do some image format checks.
+
+    Verifies that the src_format matches what qemu-img thinks the image
+    format is, and does some vmdk subformat checks.  See Bug #1996188.
+
+    - Does not check for a qcow2 backing file.
+    - Will make a call out to qemu_img if data is None.
+
+    :param source: filename of the image to check
+    :param src_format: source image format recognized by qemu_img, or None
+    :param image_id: the image ID if this is a Glance image, or None
+    :param data: a imageutils.QemuImgInfo object from this image, or None
+    :param run_as_root: when 'data' is None, call 'qemu-img info' as root
+    :raises ImageUnacceptable: when the image fails some format checks
+    :raises ProcessExecutionError: if 'qemu-img info' fails
+    """
+    if image_id is None:
+        image_id = 'internal image'
+    if data is None:
+        data = qemu_img_info(source, run_as_root=run_as_root)
+
+    if data.file_format is None:
+        raise exception.ImageUnacceptable(
+            reason=_("'qemu-img info' parsing failed."),
+            image_id=image_id)
+
+    if src_format is not None:
+        if src_format.lower() == 'ami':
+            # qemu-img doesn't recognize AMI format; see change Icde4c0f936ce.
+            # We also use lower() here (though nowhere else) to be consistent
+            # with that change.
+            pass
+        elif data.file_format != src_format:
+            LOG.debug("Rejecting image %(image_id)s due to format mismatch. "
+                      "src_format: '%(src)s', but qemu-img info reports: "
+                      "'%(qemu)s'",
+                      {'image_id': image_id,
+                       'src': src_format,
+                       'qemu': data.file_format})
+            msg = _("The image format was claimed to be '%(src)s' but the "
+                    "image data appears to be in a different format.")
+            raise exception.ImageUnacceptable(
+                image_id=image_id,
+                reason=(msg % {'src': src_format}))
+
+    if data.file_format == 'vmdk':
+        check_vmdk_image(image_id, data)
+
+
+def fetch_verify_image(context: context.RequestContext,
+                       image_service: glance.GlanceImageService,
+                       image_id: str,
+                       dest: str) -> None:
     fetch(context, image_service, image_id, dest,
           None, None)
     image_meta = image_service.show(context, image_id)
@@ -582,9 +774,12 @@ def fetch_verify_image(context, image_service, image_id, dest,
         except TypeError:
             format_raw = False
         data = get_qemu_data(image_id, has_meta, format_raw,
-                             dest, run_as_root)
+                             dest, True)
         # We can only really do verification of the image if we have
-        # qemu data to use
+        # qemu data to use.
+        # NOTE: We won't have data if qemu_img is not installed *and* the
+        # disk_format recorded in Glance is raw (otherwise an ImageUnacceptable
+        # would have been raised already).  So this isn't as bad as it looks.
         if data is not None:
             fmt = data.file_format
             if fmt is None:
@@ -599,37 +794,80 @@ def fetch_verify_image(context, image_service, image_id, dest,
                     reason=(_("fmt=%(fmt)s backed by: %(backing_file)s") %
                             {'fmt': fmt, 'backing_file': backing_file}))
 
-            # NOTE(xqueralt): If the image virtual size doesn't fit in the
-            # requested volume there is no point on resizing it because it will
-            # generate an unusable image.
-            if size is not None:
-                check_virtual_size(data.virtual_size, size, image_id)
+            # a VMDK can have a backing file, but we have to check for
+            # it differently
+            if fmt == 'vmdk':
+                check_vmdk_image(image_id, data)
 
 
-def fetch_to_vhd(context, image_service,
-                 image_id, dest, blocksize, volume_subformat=None,
-                 user_id=None, project_id=None, run_as_root=True):
+def fetch_to_vhd(context: context.RequestContext,
+                 image_service: glance.GlanceImageService,
+                 image_id: str,
+                 dest: str,
+                 blocksize: int,
+                 volume_subformat: Optional[str] = None,
+                 user_id: Optional[str] = None,
+                 project_id: Optional[str] = None,
+                 run_as_root: bool = True) -> None:
     fetch_to_volume_format(context, image_service, image_id, dest, 'vpc',
                            blocksize, volume_subformat=volume_subformat,
                            user_id=user_id, project_id=project_id,
                            run_as_root=run_as_root)
 
 
-def fetch_to_raw(context, image_service,
-                 image_id, dest, blocksize,
-                 user_id=None, project_id=None, size=None, run_as_root=True):
+def fetch_to_raw(context: context.RequestContext,
+                 image_service: glance.GlanceImageService,
+                 image_id: str,
+                 dest: str,
+                 blocksize: int,
+                 user_id: Optional[str] = None,
+                 project_id: Optional[str] = None,
+                 size: Optional[int] = None,
+                 run_as_root: bool = True) -> None:
     fetch_to_volume_format(context, image_service, image_id, dest, 'raw',
                            blocksize, user_id=user_id, project_id=project_id,
                            size=size, run_as_root=run_as_root)
 
 
-def fetch_to_volume_format(context, image_service,
-                           image_id, dest, volume_format, blocksize,
-                           volume_subformat=None, user_id=None,
-                           project_id=None, size=None, run_as_root=True):
+def check_image_conversion_disable(disk_format, volume_format, image_id,
+                                   upload=False):
+    if CONF.image_conversion_disable and disk_format != volume_format:
+        if upload:
+            msg = _("Image conversion is disabled. The image disk_format "
+                    "you have requested is '%(disk_format)s', but your "
+                    "volume can only be uploaded in the format "
+                    "'%(volume_format)s'.")
+        else:
+            msg = _("Image conversion is disabled. The volume type you have "
+                    "requested requires that the image it is being created "
+                    "from be in '%(volume_format)s' format, but the image "
+                    "you are using has the disk_format property "
+                    "'%(disk_format)s'. You must use an image with the "
+                    "disk_format property '%(volume_format)s' to create a "
+                    "volume of this type.")
+        raise exception.ImageConversionNotAllowed(
+            reason=msg % {'disk_format': disk_format,
+                          'volume_format': volume_format},
+            image_id=image_id)
+
+
+def fetch_to_volume_format(context: context.RequestContext,
+                           image_service: glance.GlanceImageService,
+                           image_id: str,
+                           dest: str,
+                           volume_format: str,
+                           blocksize: int,
+                           volume_subformat: Optional[str] = None,
+                           user_id: Optional[str] = None,
+                           project_id: Optional[str] = None,
+                           size: Optional[int] = None,
+                           run_as_root: bool = True) -> None:
     qemu_img = True
     image_meta = image_service.show(context, image_id)
 
+    check_image_conversion_disable(
+        image_meta['disk_format'], volume_format, image_id, upload=False)
+
     allow_image_compression = CONF.allow_compression_on_image_upload
     if image_meta and (image_meta.get('container_format') == 'compressed'):
         if allow_image_compression is False:
@@ -651,6 +889,10 @@ def fetch_to_volume_format(context, image_service,
             format_raw = True if image_meta['disk_format'] == 'raw' else False
         except TypeError:
             format_raw = False
+
+        # Probe using the empty tmp file to see if qemu-img is available.
+        # If it's not, and the disk_format recorded in Glance is not 'raw',
+        # this will raise ImageUnacceptable
         data = get_qemu_data(image_id, has_meta, format_raw,
                              tmp, run_as_root)
         if data is None:
@@ -663,6 +905,21 @@ def fetch_to_volume_format(context, image_service,
         else:
             fetch(context, image_service, image_id, tmp, user_id, project_id)
 
+        # NOTE(ZhengMa): This is used to do image decompression on image
+        # downloading with 'compressed' container_format. It is a
+        # transparent level between original image downloaded from
+        # Glance and Cinder image service. So the source file path is
+        # the same with destination file path.
+        if image_meta.get('container_format') == 'compressed':
+            LOG.debug("Found image with compressed container format")
+            if not accelerator.is_gzip_compressed(tmp):
+                raise exception.ImageUnacceptable(
+                    image_id=image_id,
+                    reason=_("Unsupported compressed image format found. "
+                             "Only gzip is supported currently"))
+            accel = accelerator.ImageAccel(tmp, tmp)
+            accel.decompress_img(run_as_root=run_as_root)
+
         if is_xenserver_format(image_meta):
             replace_xenserver_image_with_coalesced_vhd(tmp)
 
@@ -698,21 +955,6 @@ def fetch_to_volume_format(context, image_service,
                 reason=_("fmt=%(fmt)s backed by:%(backing_file)s")
                 % {'fmt': fmt, 'backing_file': backing_file, })
 
-        # NOTE(ZhengMa): This is used to do image decompression on image
-        # downloading with 'compressed' container_format. It is a
-        # transparent level between original image downloaded from
-        # Glance and Cinder image service. So the source file path is
-        # the same with destination file path.
-        if image_meta.get('container_format') == 'compressed':
-            LOG.debug("Found image with compressed container format")
-            if not accelerator.is_gzip_compressed(tmp):
-                raise exception.ImageUnacceptable(
-                    image_id=image_id,
-                    reason=_("Unsupported compressed image format found. "
-                             "Only gzip is supported currently"))
-            accel = accelerator.ImageAccel(tmp, tmp)
-            accel.decompress_img(run_as_root=run_as_root)
-
         # NOTE(jdg): I'm using qemu-img convert to write
         # to the volume regardless if it *needs* conversion or not
         # TODO(avishay): We can speed this up by checking if the image is raw
@@ -720,30 +962,39 @@ def fetch_to_volume_format(context, image_service,
         # check via 'qemu-img info' that what we copied was in fact a raw
         # image and not a different format with a backing file, which may be
         # malicious.
+        # FIXME: revisit the above 2 comments.  We already have an exception
+        # above for RAW format images when qemu-img is not available, and I'm
+        # pretty sure that the backing file exploit only happens when
+        # converting from some format that supports a backing file TO raw ...
+        # a bit-for-bit copy of a qcow2 with backing file will copy the backing
+        # file *reference* but not its *content*.
         disk_format = fixup_disk_format(image_meta['disk_format'])
         LOG.debug("%s was %s, converting to %s", image_id, fmt, volume_format)
 
         convert_image(tmp, dest, volume_format,
                       out_subformat=volume_subformat,
                       src_format=disk_format,
-                      run_as_root=run_as_root)
+                      run_as_root=run_as_root,
+                      image_id=image_id,
+                      data=data)
 
 
-def _validate_file_format(image_data, expected_format):
-    if image_data.file_format == expected_format:
-        return True
-    elif image_data.file_format == 'vpc' and expected_format == 'vhd':
-        # qemu-img still uses the legacy 'vpc' name for the vhd format.
-        return True
-    return False
-
-
-def upload_volume(context, image_service, image_meta, volume_path,
-                  volume_format='raw', run_as_root=True, compress=True,
-                  store_id=None, base_image_ref=None):
+def upload_volume(context: context.RequestContext,
+                  image_service: glance.GlanceImageService,
+                  image_meta: dict,
+                  volume_path: str,
+                  volume_format: str = 'raw',
+                  run_as_root: bool = True,
+                  compress: bool = True,
+                  store_id: Optional[str] = None,
+                  base_image_ref: Optional[str] = None) -> None:
     # NOTE: You probably want to use volume_utils.upload_volume(),
     # not this function.
     image_id = image_meta['id']
+
+    check_image_conversion_disable(
+        image_meta['disk_format'], volume_format, image_id, upload=True)
+
     if image_meta.get('container_format') != 'compressed':
         if (image_meta['disk_format'] == volume_format):
             LOG.debug("%s was %s, no need to convert to %s",
@@ -783,7 +1034,9 @@ def upload_volume(context, image_service, image_meta, volume_path,
         out_format = fixup_disk_format(image_meta['disk_format'])
         convert_image(volume_path, tmp, out_format,
                       run_as_root=run_as_root,
-                      compress=compress)
+                      compress=compress,
+                      image_id=image_id,
+                      data=data)
 
         data = qemu_img_info(tmp, run_as_root=run_as_root)
         if data.file_format != out_format:
@@ -807,7 +1060,9 @@ def upload_volume(context, image_service, image_meta, volume_path,
                                  base_image_ref=base_image_ref)
 
 
-def check_virtual_size(virtual_size, volume_size, image_id):
+def check_virtual_size(virtual_size: float,
+                       volume_size: int,
+                       image_id: str) -> int:
     virtual_size = int(math.ceil(float(virtual_size) / units.Gi))
 
     if virtual_size > volume_size:
@@ -821,7 +1076,7 @@ def check_virtual_size(virtual_size, volume_size, image_id):
     return virtual_size
 
 
-def check_available_space(dest, image_size, image_id):
+def check_available_space(dest: str, image_size: int, image_id: str) -> None:
     # TODO(e0ne): replace psutil with shutil.disk_usage when we drop
     # Python 2.7 support.
     if not os.path.isdir(dest):
@@ -837,50 +1092,51 @@ def check_available_space(dest, image_size, image_id):
         raise exception.ImageTooBig(image_id=image_id, reason=msg)
 
 
-def is_xenserver_format(image_meta):
+def is_xenserver_format(image_meta: dict) -> bool:
     return (
         image_meta['disk_format'] == 'vhd'
         and image_meta['container_format'] == 'ovf'
     )
 
 
-def set_vhd_parent(vhd_path, parentpath):
+def set_vhd_parent(vhd_path: str, parentpath: str) -> None:
     utils.execute('vhd-util', 'modify', '-n', vhd_path, '-p', parentpath)
 
 
-def extract_targz(archive_name, target):
+def extract_targz(archive_name: str, target: str) -> None:
     utils.execute('tar', '-xzf', archive_name, '-C', target)
 
 
-def fix_vhd_chain(vhd_chain):
+def fix_vhd_chain(vhd_chain: list[str]) -> None:
     for child, parent in zip(vhd_chain[:-1], vhd_chain[1:]):
         set_vhd_parent(child, parent)
 
 
-def get_vhd_size(vhd_path):
+def get_vhd_size(vhd_path: str) -> int:
     out, _err = utils.execute('vhd-util', 'query', '-n', vhd_path, '-v')
     return int(out)
 
 
-def resize_vhd(vhd_path, size, journal):
+def resize_vhd(vhd_path: str, size: int, journal: str) -> None:
     utils.execute(
         'vhd-util', 'resize', '-n', vhd_path, '-s', '%d' % size, '-j', journal)
 
 
-def coalesce_vhd(vhd_path):
+def coalesce_vhd(vhd_path: str) -> None:
     utils.execute(
         'vhd-util', 'coalesce', '-n', vhd_path)
 
 
-def create_temporary_file(*args, **kwargs):
+def create_temporary_file(*args: str, **kwargs: str) -> str:
     fileutils.ensure_tree(CONF.image_conversion_dir)
 
-    fd, tmp = tempfile.mkstemp(dir=CONF.image_conversion_dir, *args, **kwargs)
+    fd, tmp = tempfile.mkstemp(dir=CONF.image_conversion_dir,
+                               *args, **kwargs)  # type: ignore
     os.close(fd)
     return tmp
 
 
-def cleanup_temporary_file(backend_name):
+def cleanup_temporary_file(backend_name: str) -> None:
     temp_dir = CONF.image_conversion_dir
     if (not temp_dir or not os.path.exists(temp_dir)):
         LOG.debug("Configuration image_conversion_dir is None or the path "
@@ -904,7 +1160,7 @@ def cleanup_temporary_file(backend_name):
 
 
 @contextlib.contextmanager
-def temporary_file(*args, **kwargs):
+def temporary_file(*args: str, **kwargs) -> Generator[str, None, None]:
     tmp = None
     try:
         tmp = create_temporary_file(*args, **kwargs)
@@ -914,13 +1170,13 @@ def temporary_file(*args, **kwargs):
             fileutils.delete_if_exists(tmp)
 
 
-def temporary_dir():
+def temporary_dir() -> ContextManager[str]:
     fileutils.ensure_tree(CONF.image_conversion_dir)
 
     return utils.tempdir(dir=CONF.image_conversion_dir)
 
 
-def coalesce_chain(vhd_chain):
+def coalesce_chain(vhd_chain: list[str]) -> str:
     for child, parent in zip(vhd_chain[:-1], vhd_chain[1:]):
         with temporary_dir() as directory_for_journal:
             size = get_vhd_size(child)
@@ -932,7 +1188,7 @@ def coalesce_chain(vhd_chain):
     return vhd_chain[-1]
 
 
-def discover_vhd_chain(directory):
+def discover_vhd_chain(directory: str) -> list[str]:
     counter = 0
     chain = []
 
@@ -947,7 +1203,7 @@ def discover_vhd_chain(directory):
     return chain
 
 
-def replace_xenserver_image_with_coalesced_vhd(image_file):
+def replace_xenserver_image_with_coalesced_vhd(image_file: str) -> None:
     with temporary_dir() as tempdir:
         extract_targz(image_file, tempdir)
         chain = discover_vhd_chain(tempdir)
@@ -957,14 +1213,21 @@ def replace_xenserver_image_with_coalesced_vhd(image_file):
         os.rename(coalesced, image_file)
 
 
-def decode_cipher(cipher_spec, key_size):
+def decode_cipher(cipher_spec: str, key_size: int) -> dict[str, str]:
     """Decode a dm-crypt style cipher specification string
 
-       The assumed format being cipher[:keycount]-chainmode-ivmode[:ivopts] as
-       documented under linux/Documentation/device-mapper/dm-crypt.txt in the
-       kernel source tree.
+       The assumed format being cipher-chainmode-ivmode, similar to that
+       documented under
+       linux/Documentation/admin-guide/device-mapper/dm-crypt.txt in the
+       kernel source tree.  Cinder does not support the [:keycount] or
+       [:ivopts] options.
     """
-    cipher_alg, cipher_mode, ivgen_alg = cipher_spec.split('-')
+    try:
+        cipher_alg, cipher_mode, ivgen_alg = cipher_spec.split('-')
+    except ValueError:
+        raise exception.InvalidVolumeType(
+            reason="Invalid cipher field in encryption type")
+
     cipher_alg = cipher_alg + '-' + str(key_size)
 
     return {'cipher_alg': cipher_alg,
@@ -981,13 +1244,14 @@ class TemporaryImages(object):
     This is useful to inspect image contents before conversion.
     """
 
-    def __init__(self, image_service):
-        self.temporary_images = {}
+    def __init__(self, image_service: glance.GlanceImageService):
+        self.temporary_images: dict[str, dict] = {}
         self.image_service = image_service
         image_service.temp_images = self
 
     @staticmethod
-    def for_image_service(image_service):
+    def for_image_service(
+            image_service: glance.GlanceImageService) -> 'TemporaryImages':
         instance = image_service.temp_images
         if instance:
             return instance
@@ -995,7 +1259,11 @@ class TemporaryImages(object):
 
     @classmethod
     @contextlib.contextmanager
-    def fetch(cls, image_service, context, image_id, suffix=''):
+    def fetch(cls,
+              image_service: glance.GlanceImageService,
+              context: context.RequestContext,
+              image_id: str,
+              suffix: Optional[str] = '') -> Generator[str, None, None]:
         tmp_images = cls.for_image_service(image_service).temporary_images
         with temporary_file(prefix='image_fetch_%s_' % image_id,
                             suffix=suffix) as tmp:
@@ -1011,7 +1279,7 @@ class TemporaryImages(object):
         LOG.debug("Temporary image %(id)s for user %(user)s is deleted.",
                   {'id': image_id, 'user': user})
 
-    def get(self, context, image_id):
+    def get(self, context: context.RequestContext, image_id: str):
         user = context.user_id
         if not self.temporary_images.get(user):
             return None
