@@ -14,14 +14,17 @@
 #    under the License.
 """Unit tests for INFINIDAT InfiniBox volume driver."""
 
+import copy
 import functools
 import platform
 import socket
 from unittest import mock
 
+import ddt
 from oslo_utils import units
 
 from cinder import exception
+from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import test
 from cinder import version
 from cinder.volume import configuration
@@ -47,12 +50,20 @@ TEST_TARGET_PORTAL3 = '{}:{}'.format(TEST_IP_ADDRESS3, TEST_ISCSI_TCP_PORT2)
 TEST_TARGET_PORTAL4 = '{}:{}'.format(TEST_IP_ADDRESS4, TEST_ISCSI_TCP_PORT2)
 TEST_FC_PROTOCOL = 'fc'
 TEST_ISCSI_PROTOCOL = 'iscsi'
+TEST_VOLUME_SOURCE_NAME = 'test-volume'
+TEST_VOLUME_SOURCE_ID = 12345
+TEST_VOLUME_METADATA = {'cinder_id': fake.VOLUME_ID}
+TEST_SNAPSHOT_SOURCE_NAME = 'test-snapshot'
+TEST_SNAPSHOT_SOURCE_ID = 67890
+TEST_SNAPSHOT_METADATA = {'cinder_id': fake.SNAPSHOT_ID}
 
-test_volume = mock.Mock(id=1, size=1, volume_type_id=1)
-test_snapshot = mock.Mock(id=2, volume=test_volume, volume_id='1')
-test_clone = mock.Mock(id=3, size=1)
-test_group = mock.Mock(id=4)
-test_snapgroup = mock.Mock(id=5, group=test_group)
+test_volume = mock.Mock(id=fake.VOLUME_ID, size=1,
+                        volume_type_id=fake.VOLUME_TYPE_ID)
+test_snapshot = mock.Mock(id=fake.SNAPSHOT_ID, volume=test_volume,
+                          volume_id=test_volume.id)
+test_clone = mock.Mock(id=fake.VOLUME4_ID, size=1)
+test_group = mock.Mock(id=fake.GROUP_ID)
+test_snapgroup = mock.Mock(id=fake.GROUP_SNAPSHOT_ID, group=test_group)
 test_connector = dict(wwpns=[TEST_WWN_1],
                       initiator=TEST_INITIATOR_IQN)
 
@@ -78,25 +89,15 @@ class InfiniboxDriverTestCaseBase(test.TestCase):
     def setUp(self):
         super(InfiniboxDriverTestCaseBase, self).setUp()
 
-        # create mock configuration
-        self.configuration = mock.Mock(spec=configuration.Configuration)
-        self.configuration.infinidat_storage_protocol = TEST_FC_PROTOCOL
-        self.configuration.san_ip = 'mockbox'
-        self.configuration.infinidat_pool_name = 'mockpool'
-        self.configuration.san_thin_provision = True
-        self.configuration.san_login = 'user'
-        self.configuration.san_password = 'pass'
-        self.configuration.volume_backend_name = 'mock'
-        self.configuration.volume_dd_blocksize = '1M'
-        self.configuration.use_multipath_for_image_xfer = False
-        self.configuration.enforce_multipath_for_image_xfer = False
-        self.configuration.num_volume_device_scan_tries = 1
-        self.configuration.san_is_local = False
-        self.configuration.chap_username = None
-        self.configuration.chap_password = None
-        self.configuration.infinidat_use_compression = None
-        self.configuration.max_over_subscription_ratio = 10.0
-
+        self.configuration = configuration.Configuration(None)
+        self.configuration.append_config_values(infinidat.infinidat_opts)
+        self.override_config('san_ip', 'infinibox',
+                             configuration.SHARED_CONF_GROUP)
+        self.override_config('san_login', 'user',
+                             configuration.SHARED_CONF_GROUP)
+        self.override_config('san_password', 'password',
+                             configuration.SHARED_CONF_GROUP)
+        self.override_config('infinidat_pool_name', 'pool')
         self.driver = infinidat.InfiniboxVolumeDriver(
             configuration=self.configuration)
         self._system = self._infinibox_mock()
@@ -118,16 +119,26 @@ class InfiniboxDriverTestCaseBase(test.TestCase):
     def _infinibox_mock(self):
         result = mock.Mock()
         self._mock_volume = mock.Mock()
+        self._mock_new_volume = mock.Mock()
+        self._mock_volume.get_id.return_value = TEST_VOLUME_SOURCE_ID
+        self._mock_volume.get_name.return_value = TEST_VOLUME_SOURCE_NAME
+        self._mock_volume.get_type.return_value = 'MASTER'
+        self._mock_volume.get_pool_name.return_value = (
+            self.configuration.infinidat_pool_name)
         self._mock_volume.get_size.return_value = 1 * units.Gi
         self._mock_volume.has_children.return_value = False
+        self._mock_volume.get_qos_policy.return_value = None
         self._mock_volume.get_logical_units.return_value = []
+        self._mock_volume.get_all_metadata.return_value = {}
         self._mock_volume.create_snapshot.return_value = self._mock_volume
+        self._mock_snapshot = mock.Mock()
         self._mock_host = mock.Mock()
         self._mock_host.get_luns.return_value = []
         self._mock_host.map_volume().get_lun.return_value = TEST_LUN
         self._mock_pool = mock.Mock()
         self._mock_pool.get_free_physical_capacity.return_value = units.Gi
         self._mock_pool.get_physical_capacity.return_value = units.Gi
+        self._mock_pool.get_volumes.return_value = [self._mock_volume]
         self._mock_name_space1 = mock.Mock()
         self._mock_name_space2 = mock.Mock()
         self._mock_name_space1.get_ips.return_value = [
@@ -157,6 +168,7 @@ class InfiniboxDriverTestCaseBase(test.TestCase):
         raise FakeInfinisdkException()
 
 
+@ddt.ddt
 class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
     def _generate_mock_object_metadata(self, cinder_object):
         return {"system": "openstack",
@@ -195,6 +207,16 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
             infinidat._INFINIDAT_CINDER_IDENTIFIER)
         self._system.login.assert_called_once()
 
+    @mock.patch('cinder.volume.drivers.infinidat.infinisdk.InfiniBox')
+    @ddt.data(True, False)
+    def test_ssl_options(self, use_ssl, infinibox):
+        auth = (self.configuration.san_login,
+                self.configuration.san_password)
+        self.override_config('driver_use_ssl', use_ssl)
+        self.driver.do_setup(None)
+        infinibox.assert_called_once_with(self.configuration.san_ip,
+                                          auth=auth, use_ssl=use_ssl)
+
     def test_initialize_connection(self):
         self._system.hosts.safe_get.return_value = None
         result = self.driver.initialize_connection(test_volume, test_connector)
@@ -214,7 +236,7 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
 
     def test_initialize_connection_volume_doesnt_exist(self):
         self._system.volumes.safe_get.return_value = None
-        self.assertRaises(exception.InvalidVolume,
+        self.assertRaises(exception.VolumeNotFound,
                           self.driver.initialize_connection,
                           test_volume, test_connector)
 
@@ -249,7 +271,7 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
 
     def test_terminate_connection_volume_doesnt_exist(self):
         self._system.volumes.safe_get.return_value = None
-        self.assertRaises(exception.InvalidVolume,
+        self.assertRaises(exception.VolumeNotFound,
                           self.driver.terminate_connection,
                           test_volume, test_connector)
 
@@ -277,9 +299,10 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
                           self.driver.get_volume_stats)
 
     def test_get_volume_stats_max_over_subscription_ratio(self):
+        self.override_config('san_thin_provision', True)
+        self.override_config('max_over_subscription_ratio', 10.0)
         result = self.driver.get_volume_stats()
-        # check the defaults defined in setUp
-        self.assertEqual(10.0, result['max_over_subscription_ratio'])
+        self.assertEqual('10.0', result['max_over_subscription_ratio'])
         self.assertTrue(result['thin_provisioning_support'])
         self.assertFalse(result['thick_provisioning_support'])
 
@@ -304,7 +327,7 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
 
     @mock.patch("cinder.volume.volume_types.get_volume_type_qos_specs")
     def test_create_volume_compression_enabled(self, *mocks):
-        self.configuration.infinidat_use_compression = True
+        self.override_config('infinidat_use_compression', True)
         self.driver.create_volume(test_volume)
         self.assertTrue(
             self._system.volumes.create.call_args[1]["compression_enabled"]
@@ -312,7 +335,7 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
 
     @mock.patch("cinder.volume.volume_types.get_volume_type_qos_specs")
     def test_create_volume_compression_not_enabled(self, *mocks):
-        self.configuration.infinidat_use_compression = False
+        self.override_config('infinidat_use_compression', False)
         self.driver.create_volume(test_volume)
         self.assertFalse(
             self._system.volumes.create.call_args[1]["compression_enabled"]
@@ -359,7 +382,7 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
 
     def test_create_snapshot_volume_doesnt_exist(self):
         self._system.volumes.safe_get.return_value = None
-        self.assertRaises(exception.InvalidVolume,
+        self.assertRaises(exception.VolumeNotFound,
                           self.driver.create_snapshot, test_snapshot)
 
     def test_create_snapshot_api_fail(self):
@@ -377,7 +400,7 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
 
     def test_create_volume_from_snapshot_doesnt_exist(self):
         self._system.volumes.safe_get.return_value = None
-        self.assertRaises(exception.InvalidSnapshot,
+        self.assertRaises(exception.SnapshotNotFound,
                           self.driver.create_volume_from_snapshot,
                           test_clone, test_snapshot)
 
@@ -597,6 +620,244 @@ class InfiniboxDriverTestCase(InfiniboxDriverTestCaseBase):
         self._mock_host.unmap_volume.assert_called_once()
         self._mock_host.safe_delete.assert_called_once()
 
+    def test_snapshot_revert_use_temp_snapshot(self):
+        result = self.driver.snapshot_revert_use_temp_snapshot()
+        self.assertFalse(result)
+
+    @ddt.data((1, 1), (1, 2))
+    @ddt.unpack
+    def test_revert_to_snapshot_resize(self, volume_size, snapshot_size):
+        volume = copy.deepcopy(test_volume)
+        snapshot = copy.deepcopy(test_snapshot)
+        snapshot.volume.size = snapshot_size
+        self._system.volumes.safe_get.side_effect = [self._mock_snapshot,
+                                                     self._mock_volume,
+                                                     self._mock_volume]
+        self._mock_volume.get_size.side_effect = [volume_size * units.Gi,
+                                                  volume_size * units.Gi]
+        self.driver.revert_to_snapshot(None, volume, snapshot)
+        self._mock_volume.restore.assert_called_once_with(self._mock_snapshot)
+        if volume_size == snapshot_size:
+            self._mock_volume.resize.assert_not_called()
+        else:
+            delta = (snapshot_size - volume_size) * units.Gi
+            self._mock_volume.resize.assert_called_with(delta)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    def test_manage_existing_by_source_name(self, *mocks):
+        existing_ref = {'source-name': TEST_VOLUME_SOURCE_NAME}
+        self.driver.manage_existing(test_volume, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    def test_manage_existing_by_source_id(self, *mocks):
+        existing_ref = {'source-id': TEST_VOLUME_SOURCE_ID}
+        self.driver.manage_existing(test_volume, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    def test_manage_existing_by_invalid_source(self, *mocks):
+        existing_ref = {'source-path': None}
+        self.assertRaises(exception.ManageExistingInvalidReference,
+                          self.driver.manage_existing,
+                          test_volume, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    @mock.patch('cinder.volume.volume_utils.check_already_managed_volume',
+                return_value=False)
+    def test_manage_existing_not_managed(self, *mocks):
+        self._mock_volume.get_all_metadata.return_value = (
+            TEST_VOLUME_METADATA)
+        existing_ref = {'source-name': TEST_VOLUME_SOURCE_NAME}
+        self.driver.manage_existing(test_volume, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    @mock.patch('cinder.volume.volume_utils.check_already_managed_volume',
+                return_value=True)
+    def test_manage_existing_already_managed(self, *mocks):
+        self._mock_volume.get_all_metadata.return_value = (
+            TEST_VOLUME_METADATA)
+        existing_ref = {'source-name': TEST_VOLUME_SOURCE_NAME}
+        self.assertRaises(exception.ManageExistingAlreadyManaged,
+                          self.driver.manage_existing,
+                          test_volume, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    def test_manage_existing_invalid_pool(self, *mocks):
+        existing_ref = {'source-name': TEST_VOLUME_SOURCE_NAME}
+        self._mock_volume.get_pool_name.return_value = 'invalid'
+        self.assertRaises(exception.InvalidConfigurationValue,
+                          self.driver.manage_existing,
+                          test_volume, existing_ref)
+
+    def test_manage_existing_get_size(self):
+        existing_ref = {'source-name': TEST_VOLUME_SOURCE_NAME}
+        size = self.driver.manage_existing_get_size(test_volume, existing_ref)
+        self.assertEqual(test_volume.size, size)
+
+    def test_get_manageable_volumes(self):
+        cinder_volumes = [test_volume]
+        self._mock_volume.is_snapshot.return_value = False
+        self._mock_volume.get_all_metadata.return_value = {
+            'cinder_id': fake.VOLUME2_ID
+        }
+        self.driver.get_manageable_volumes(cinder_volumes, None,
+                                           1, 0, [], [])
+
+    def test_get_manageable_volumes_already_managed(self):
+        cinder_volumes = [test_volume]
+        self._mock_volume.get_id.return_value = TEST_VOLUME_SOURCE_ID
+        self._mock_volume.get_all_metadata.return_value = (
+            TEST_VOLUME_METADATA)
+        self._mock_volume.is_snapshot.return_value = False
+        self.driver.get_manageable_volumes(cinder_volumes, None,
+                                           1, 0, [], [])
+
+    def test_get_manageable_volumes_but_snapshots(self):
+        cinder_volumes = [test_volume]
+        self._mock_volume.is_snapshot.return_value = True
+        self.driver.get_manageable_volumes(cinder_volumes, None,
+                                           1, 0, [], [])
+
+    def test_get_manageable_volumes_has_mappings(self):
+        cinder_volumes = [test_volume]
+        self._mock_volume.is_snapshot.return_value = False
+        self._mock_volume.get_all_metadata.return_value = {
+            'cinder_id': fake.VOLUME2_ID
+        }
+        lun = mock.Mock()
+        self._mock_volume.get_logical_units.return_value = [lun]
+        self.driver.get_manageable_volumes(cinder_volumes, None,
+                                           1, 0, [], [])
+
+    def test_get_manageable_volumes_has_snapshots(self):
+        cinder_volumes = [test_volume]
+        self._mock_volume.is_snapshot.return_value = False
+        self._mock_volume.has_children.return_value = True
+        self._mock_volume.get_all_metadata.return_value = {
+            'cinder_id': fake.VOLUME2_ID
+        }
+        self.driver.get_manageable_volumes(cinder_volumes, None,
+                                           1, 0, [], [])
+
+    def test_unmanage(self):
+        self.driver.unmanage(test_volume)
+
+    @mock.patch('cinder.objects.Snapshot.exists', return_value=True)
+    def test__check_already_managed_snapshot(self, *mocks):
+        self.driver._check_already_managed_snapshot(test_snapshot.id)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    def test_manage_existing_snapshot_by_source_name(self, *mocks):
+        existing_ref = {'source-name': TEST_SNAPSHOT_SOURCE_NAME}
+        self.driver.manage_existing_snapshot(test_snapshot, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    def test_manage_existing_snapshot_by_source_id(self, *mocks):
+        existing_ref = {'source-id': TEST_SNAPSHOT_SOURCE_ID}
+        self.driver.manage_existing_snapshot(test_snapshot, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    def test_manage_existing_snapshot_but_volume(self, *mocks):
+        existing_ref = {'source-id': TEST_SNAPSHOT_SOURCE_ID}
+        self._mock_volume.is_snapshot.return_value = False
+        self.assertRaises(exception.InvalidSnapshot,
+                          self.driver.manage_existing_snapshot,
+                          test_snapshot, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    def test_manage_existing_snapshot_by_invalid_source(self, *mocks):
+        existing_ref = {'source-path': None}
+        self.assertRaises(exception.ManageExistingInvalidReference,
+                          self.driver.manage_existing_snapshot,
+                          test_snapshot, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    def test_manage_existing_snapshot_by_non_cinder_id(self, *mocks):
+        self._mock_volume.get_all_metadata.return_value = {'cinder_id': 'x'}
+        existing_ref = {'source-id': TEST_SNAPSHOT_SOURCE_ID}
+        self.driver.manage_existing_snapshot(test_snapshot, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_check_already_managed_snapshot', return_value=False)
+    def test_manage_existing_snapshot_not_managed(self, *mocks):
+        self._mock_volume.get_all_metadata.return_value = (
+            TEST_SNAPSHOT_METADATA)
+        existing_ref = {'source-name': TEST_SNAPSHOT_SOURCE_NAME}
+        self.driver.manage_existing(test_snapshot, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    @mock.patch('cinder.volume.drivers.infinidat.InfiniboxVolumeDriver.'
+                '_check_already_managed_snapshot', return_value=True)
+    def test_manage_existing_snapshot_already_managed(self, *mocks):
+        self._mock_volume.get_all_metadata.return_value = (
+            TEST_SNAPSHOT_METADATA)
+        existing_ref = {'source-name': TEST_SNAPSHOT_SOURCE_NAME}
+        self.assertRaises(exception.ManageExistingAlreadyManaged,
+                          self.driver.manage_existing_snapshot,
+                          test_snapshot, existing_ref)
+
+    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
+    def test_manage_existing_snapshot_invalid_pool(self, *mocks):
+        existing_ref = {'source-name': TEST_SNAPSHOT_SOURCE_NAME}
+        self._mock_volume.get_pool_name.return_value = 'invalid'
+        self.assertRaises(exception.InvalidConfigurationValue,
+                          self.driver.manage_existing_snapshot,
+                          test_snapshot, existing_ref)
+
+    def test_manage_existing_snapshot_get_size(self):
+        existing_ref = {'source-name': TEST_SNAPSHOT_SOURCE_NAME}
+        size = self.driver.manage_existing_snapshot_get_size(test_volume,
+                                                             existing_ref)
+        self.assertEqual(test_snapshot.volume.size, size)
+
+    def test_get_manageable_snapshots(self):
+        cinder_snapshots = [test_snapshot]
+        self._mock_volume.is_snapshot.return_value = True
+        self._mock_volume.get_all_metadata.return_value = {
+            'cinder_id': fake.SNAPSHOT2_ID
+        }
+        self.driver.get_manageable_snapshots(cinder_snapshots,
+                                             None, 1, 0, [], [])
+
+    def test_get_manageable_snapshots_already_managed(self):
+        cinder_snapshots = [test_snapshot]
+        self._mock_volume.get_id.return_value = TEST_SNAPSHOT_SOURCE_ID
+        self._mock_volume.get_all_metadata.return_value = (
+            TEST_SNAPSHOT_METADATA)
+        self._mock_volume.is_snapshot.return_value = True
+        self.driver.get_manageable_snapshots(cinder_snapshots,
+                                             None, 1, 0, [], [])
+
+    def test_get_manageable_snapshots_but_volumes(self):
+        cinder_snapshots = [test_snapshot]
+        self._mock_volume.is_snapshot.return_value = False
+        self.driver.get_manageable_snapshots(cinder_snapshots,
+                                             None, 1, 0, [], [])
+
+    def test_get_manageable_snapshots_has_mappings(self):
+        cinder_snapshots = [test_snapshot]
+        self._mock_volume.is_snapshot.return_value = True
+        self._mock_volume.get_all_metadata.return_value = {
+            'cinder_id': fake.SNAPSHOT2_ID
+        }
+        lun = mock.Mock()
+        self._mock_volume.get_logical_units.return_value = [lun]
+        self.driver.get_manageable_snapshots(cinder_snapshots,
+                                             None, 1, 0, [], [])
+
+    def test_get_manageable_snapshots_has_clones(self):
+        cinder_snapshots = [test_snapshot]
+        self._mock_volume.is_snapshot.return_value = True
+        self._mock_volume.has_children.return_value = True
+        self._mock_volume.get_all_metadata.return_value = {
+            'cinder_id': fake.SNAPSHOT2_ID
+        }
+        self.driver.get_manageable_snapshots(cinder_snapshots,
+                                             None, 1, 0, [], [])
+
+    def test_unmanage_snapshot(self):
+        self.driver.unmanage_snapshot(test_snapshot)
+
 
 class InfiniboxDriverTestCaseFC(InfiniboxDriverTestCaseBase):
     def test_initialize_connection_multiple_wwpns(self):
@@ -615,13 +876,15 @@ class InfiniboxDriverTestCaseFC(InfiniboxDriverTestCaseBase):
 class InfiniboxDriverTestCaseISCSI(InfiniboxDriverTestCaseBase):
     def setUp(self):
         super(InfiniboxDriverTestCaseISCSI, self).setUp()
-        self.configuration.infinidat_storage_protocol = TEST_ISCSI_PROTOCOL
-        self.configuration.infinidat_iscsi_netspaces = [TEST_ISCSI_NAMESPACE1]
-        self.configuration.use_chap_auth = False
+        self.override_config('infinidat_storage_protocol',
+                             TEST_ISCSI_PROTOCOL)
+        self.override_config('infinidat_iscsi_netspaces',
+                             [TEST_ISCSI_NAMESPACE1])
+        self.override_config('use_chap_auth', False)
         self.driver.do_setup(None)
 
     def test_setup_without_netspaces_configured(self):
-        self.configuration.infinidat_iscsi_netspaces = []
+        self.override_config('infinidat_iscsi_netspaces', [])
         self.assertRaises(exception.VolumeDriverException,
                           self.driver.do_setup, None)
 
@@ -660,7 +923,7 @@ class InfiniboxDriverTestCaseISCSI(InfiniboxDriverTestCaseBase):
                           test_volume, test_connector)
 
     def test_initialize_connection_with_chap(self):
-        self.configuration.use_chap_auth = True
+        self.override_config('use_chap_auth', True)
         result = self.driver.initialize_connection(test_volume, test_connector)
         self.assertEqual(1, result['data']['target_lun'])
         self.assertEqual('CHAP', result['data']['auth_method'])
@@ -668,8 +931,8 @@ class InfiniboxDriverTestCaseISCSI(InfiniboxDriverTestCaseBase):
         self.assertIn('auth_password', result['data'])
 
     def test_initialize_connection_multiple_netspaces(self):
-        self.configuration.infinidat_iscsi_netspaces = [
-            TEST_ISCSI_NAMESPACE1, TEST_ISCSI_NAMESPACE2]
+        self.override_config('infinidat_iscsi_netspaces',
+                             [TEST_ISCSI_NAMESPACE1, TEST_ISCSI_NAMESPACE2])
         self._system.network_spaces.safe_get.side_effect = [
             self._mock_name_space1, self._mock_name_space2]
         result = self.driver.initialize_connection(test_volume, test_connector)
@@ -697,8 +960,8 @@ class InfiniboxDriverTestCaseISCSI(InfiniboxDriverTestCaseBase):
         self.assertEqual(expected, result)
 
     def test_initialize_connection_multiple_netspaces_multipath(self):
-        self.configuration.infinidat_iscsi_netspaces = [
-            TEST_ISCSI_NAMESPACE1, TEST_ISCSI_NAMESPACE2]
+        self.override_config('infinidat_iscsi_netspaces',
+                             [TEST_ISCSI_NAMESPACE1, TEST_ISCSI_NAMESPACE2])
         self._system.network_spaces.safe_get.side_effect = [
             self._mock_name_space1, self._mock_name_space2]
         self._mock_name_space1.get_ips.return_value = [
