@@ -143,9 +143,10 @@ class Lun(object):
         2.1.1 - Added support for replication consistency group.
         2.1.2 - Added support for cloning volume asynchronously.
         2.3.0 - Added support for reporting backend state.
+        2.5.0 - Added support for revert to snapshot operation.
     """
 
-    VERSION = "2.3.0"
+    VERSION = "2.5.0"
 
     class FakeLun(object):
 
@@ -222,7 +223,7 @@ class Lun(object):
             self.ds_id = provider_location['vol_hex_id']
         else:
             self.ds_id = None
-        self.cinder_name = volume.display_name
+        self.cinder_name = volume.name
         self.pool_lss_pair = {}
         self.is_snapshot = is_snapshot
         if self.is_snapshot:
@@ -230,20 +231,14 @@ class Lun(object):
                           if volume.group_snapshot else None)
             self.size = volume.volume_size
             # ds8k supports at most 16 chars
-            self.ds_name = (
-                "OS%s:%s" % ('snap', helper.filter_alnum(self.cinder_name))
-            )[:16]
+            self.ds_name = helper.filter_alnum(self.cinder_name)[:16]
             self.metadata = self._get_snapshot_metadata(volume)
             self.source_volid = volume.volume_id
         else:
             self.group = Group(volume.group) if volume.group else None
             self.size = volume.size
-            self.ds_name = (
-                "OS%s:%s" % ('vol', helper.filter_alnum(self.cinder_name))
-            )[:16]
-            self.replica_ds_name = (
-                "OS%s:%s" % ('Replica', helper.filter_alnum(self.cinder_name))
-            )[:16]
+            self.ds_name = helper.filter_alnum(self.cinder_name)[:16]
+            self.replica_ds_name = helper.filter_alnum(self.cinder_name)[:16]
             self.previous_status = volume.previous_status
             self.replication_status = volume.replication_status
             self.replication_driver_data = (
@@ -1065,6 +1060,31 @@ class DS8KProxy(proxy.IBMStorageProxy):
             volume_update = lun.get_volume_update()
         return True, volume_update
 
+    @proxy._trace_time
+    @proxy.logger
+    def revert_to_snapshot(self, context, volume, snapshot):
+        """Revert volume to snapshot."""
+        if snapshot.volume_size != volume.size:
+            raise exception.InvalidInput(
+                reason=_('Reverting volume is not supported if the volume '
+                         'size is not equal to the snapshot size.'))
+
+        vol_lun = Lun(volume)
+        snap_lun = Lun(snapshot, is_snapshot=True)
+        if vol_lun.type_replication:
+            raise exception.VolumeDriverException(
+                message=_('Driver does not support revert to snapshot '
+                          'of replicated volume.'))
+
+        try:
+            self._clone_lun(snap_lun, vol_lun)
+        except Exception as err:
+            msg = (_("Reverting volume %(vol)s to snapshot %(snap)s failed "
+                     "due to: %(err)s.")
+                   % {"vol": volume.name, "snap": snapshot.name, "err": err})
+            LOG.error(msg)
+            raise exception.VolumeBackendAPIException(data=msg)
+
     @proxy._trace_time
     @proxy.logger
     def initialize_connection(self, volume, connector, **kwargs):
@@ -1093,8 +1113,9 @@ class DS8KProxy(proxy.IBMStorageProxy):
             backend_helper = self._helper
         if isinstance(backend_helper, helper.DS8KECKDHelper):
             LOG.info('Detach the volume %s.', lun.ds_id)
-            return backend_helper.terminate_connection(lun.ds_id, connector,
-                                                       force, **kwargs)
+            return backend_helper.terminate_connection(volume, lun.ds_id,
+                                                       connector, force,
+                                                       **kwargs)
         else:
             vol_mapped, host_id, map_info = (
                 backend_helper.check_vol_mapped_to_host(connector, lun.ds_id))
@@ -1135,11 +1156,13 @@ class DS8KProxy(proxy.IBMStorageProxy):
                     else:
                         LOG.info('Detach the volume %s.', lun.replica_ds_id)
                         return backend_helper.terminate_connection(
-                            lun.replica_ds_id, host_id, connector, map_info)
+                            volume, lun.replica_ds_id, host_id, connector,
+                            map_info)
             elif host_id and vol_mapped:
                 LOG.info('Detaching volume %s.', lun.ds_id)
-                return backend_helper.terminate_connection(lun.ds_id, host_id,
-                                                           connector, map_info)
+                return backend_helper.terminate_connection(volume, lun.ds_id,
+                                                           host_id, connector,
+                                                           map_info)
 
     @proxy.logger
     def create_group(self, ctxt, group):
